import 'dart:io';
import 'package:device_info/device_info.dart';
import 'package:flutter/services.dart';

class NUIDeviceInfo{
  NUIDeviceInfo({this.model, this.version, this.brand, this.manufacturer, this.details, this.platform});
  String model;
  String version;
  String brand;
  String manufacturer;
  String platform;
  Map<String, dynamic> details;
}

enum NUIPlatform{
  ANDROID,
  IOS,
  WEB
}

extension NUIPlatformExt on NUIPlatform{
  String value(){
    switch(this){
      case NUIPlatform.ANDROID: {
        return "Android";
      }
      case NUIPlatform.IOS: {
        return "iOS";
      }
      case NUIPlatform.WEB: {
        return "Web";
      }
      default: {
        return "Unknown";
      }
    }
  }
}

Future<NUIDeviceInfo> getDeviceInfo() async {
  return NUIDeviceUtil.getDeviceInfo();
}

class NUIDeviceUtil{
  static final DeviceInfoPlugin deviceInfoPlugin = DeviceInfoPlugin();
  NUIDeviceUtil._builder();

  static Future<NUIDeviceInfo> getDeviceInfo() async {
    Map<String, dynamic> deviceData;
    NUIDeviceInfo deviceInfo;

    try {
      if (Platform.isAndroid) {
        final androidDeviceInfo = await deviceInfoPlugin.androidInfo;
        deviceData = _readAndroidBuildData(androidDeviceInfo);
        deviceInfo = NUIDeviceInfo(
          model: androidDeviceInfo.model,
          version: androidDeviceInfo.version.sdkInt.toString(),
          brand: androidDeviceInfo.brand,
          manufacturer: androidDeviceInfo.manufacturer,
          details: deviceData,
          platform: NUIPlatform.ANDROID.value()
        );
      } else if (Platform.isIOS) {
        final iosDeviceInfo = await deviceInfoPlugin.iosInfo;
        deviceData = _readIosDeviceInfo(iosDeviceInfo);
        deviceInfo = NUIDeviceInfo(
          model: iosDeviceInfo.model,
          version: iosDeviceInfo.systemVersion,
          brand: "iOS",
          manufacturer: "Apple",
          details: deviceData,
          platform: NUIPlatform.IOS.value()
        );
      }
    } on PlatformException {
      deviceData = <String, dynamic>{
        'Error:': 'Failed to get platform version.'
      };
    }
    return deviceInfo;
  }

  static Map<String, dynamic> _readAndroidBuildData(AndroidDeviceInfo build) {
    return <String, dynamic>{
      'version.securityPatch': build.version.securityPatch,
      'version.sdkInt': build.version.sdkInt,
      'version.release': build.version.release,
      'version.previewSdkInt': build.version.previewSdkInt,
      'version.incremental': build.version.incremental,
      'version.codename': build.version.codename,
      'version.baseOS': build.version.baseOS,
      'board': build.board,
      'bootloader': build.bootloader,
      'brand': build.brand,
      'device': build.device,
      'display': build.display,
      'fingerprint': build.fingerprint,
      'hardware': build.hardware,
      'host': build.host,
      'id': build.id,
      'manufacturer': build.manufacturer,
      'model': build.model,
      'product': build.product,
      'supported32BitAbis': build.supported32BitAbis,
      'supported64BitAbis': build.supported64BitAbis,
      'supportedAbis': build.supportedAbis,
      'tags': build.tags,
      'type': build.type,
      'isPhysicalDevice': build.isPhysicalDevice,
      'androidId': build.androidId,
      'systemFeatures': build.systemFeatures,
    };
  }

  static Map<String, dynamic> _readIosDeviceInfo(IosDeviceInfo data) {
    return <String, dynamic>{
      'name': data.name,
      'systemName': data.systemName,
      'systemVersion': data.systemVersion,
      'model': data.model,
      'localizedModel': data.localizedModel,
      'identifierForVendor': data.identifierForVendor,
      'isPhysicalDevice': data.isPhysicalDevice,
      'utsname.sysname:': data.utsname.sysname,
      'utsname.nodename:': data.utsname.nodename,
      'utsname.release:': data.utsname.release,
      'utsname.version:': data.utsname.version,
      'utsname.machine:': data.utsname.machine,
    };
  }
}