import 'dart:math' as math;

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';

typedef String TextMapper(String numberText);

class NUINumberStringPicker extends StatelessWidget {

  static const double kDefaultItemExtent = 50.0;
  static const double kDefaultListViewCrossAxisSize = 100.0;

  NUINumberStringPicker.string({
    Key key,
    this.textMapper,
    this.onChanged,
    @required String initialValue,
    @required this.stringSelections,
    this.itemExtent = kDefaultItemExtent,
    this.listViewWidth = kDefaultListViewCrossAxisSize,
    this.scrollDirection = Axis.vertical,
    this.highlightSelectedValue = true,
    this.decoration,
    this.haptics = false,
    this.textStyle,
    this.selectedTextStyle,
  })  : assert(initialValue != null),
        assert(scrollDirection != null),
        selectedValue = initialValue,
        intScrollController = ScrollController(

        ),
        listViewHeight = 3 * itemExtent,
        super(key: key);

  ///called when selected value changes
  final ValueChanged<String> onChanged;

  ///build the text of each item on the picker
  final TextMapper textMapper;

  ///height of every list element in pixels
  final double itemExtent;

  ///height of list view in pixels
  final double listViewHeight;

  ///width of list view in pixels
  final double listViewWidth;

  ///ScrollController used for integer list
  final ScrollController intScrollController;

  ///Currently selected string value
  final String selectedValue;

  ///If currently selected value should be highlighted
  final bool highlightSelectedValue;

  ///Decoration to apply to central box where the selected value is placed
  final Decoration decoration;

  /// Direction of scrolling
  final Axis scrollDirection;

  ///Whether to trigger haptic pulses or not
  final bool haptics;

  ///Default Style Of Numbers
  final TextStyle textStyle;

  ///Style Of Selected Number
  final TextStyle selectedTextStyle;

  final List<String> stringSelections;

  //
  //----------------------------- PUBLIC ------------------------------
  //

  /// Used to animate integer number picker to new selected value
  Future<void> animateValue(String valueToSelect) {
    final index = stringSelections.indexOf(valueToSelect);
    return animateStringToIndex(index);
  }

  Future<void> animateStringToIndex(int index) =>
      _animate(intScrollController, index * itemExtent);

  ///main widget
  @override
  Widget build(BuildContext context) {
    final ThemeData themeData = Theme.of(context);
    return _stringListView(themeData);
  }

  Widget _stringListView(ThemeData themeData) {
    TextStyle defaultStyle = textStyle ?? themeData.textTheme.bodyText2;
    TextStyle selectedStyle = selectedTextStyle ??
        themeData.textTheme.headline5.copyWith(color: themeData.accentColor);

    var listItemCount = stringSelections.length + 2;

    return Listener(
      onPointerUp: (ev) {
        ///used to detect that user stopped scrolling
        // ignore: invalid_use_of_protected_member, invalid_use_of_visible_for_testing_member
        if (intScrollController.position.activity is HoldScrollActivity) {
          animateValue(selectedValue);
        }
      },
      child: NotificationListener(
        child: Container(
          height: listViewHeight,
          width: listViewWidth,
          child: Stack(
            children: <Widget>[
              ListView.builder(
                scrollDirection: scrollDirection,
                controller: intScrollController,
                itemExtent: itemExtent,
                itemCount: listItemCount,
                cacheExtent: _calculateCacheExtent(listItemCount),
                itemBuilder: (BuildContext context, int index) {
                  final String value = _valueFromIndex(index);

                  //define special style for selected (middle) element
                  final TextStyle itemStyle =
                  value == selectedValue && highlightSelectedValue
                      ? selectedStyle
                      : defaultStyle;

                  bool isExtra = index == 0 || index == listItemCount - 1;

                  return isExtra
                      ? Container() //empty first and last element
                      : Center(
                    child: Text(
                      getDisplayedValue(value),
                      style: itemStyle,
                    ),
                  );
                },
              ),
              _NumberPickerSelectedItemDecoration(
                axis: scrollDirection,
                itemExtent: itemExtent,
                decoration: decoration,
              ),
            ],
          ),
        ),
        onNotification: _onIntegerNotification,
      ),
    );
  }

  String getDisplayedValue(String value) {
    final text = value;
    return textMapper != null ? textMapper(text) : text;
  }

  //
  // ----------------------------- LOGIC -----------------------------
  //

  String _valueFromIndex(int index) {
    return stringSelections[index];
  }

  bool _onIntegerNotification(Notification notification) {
    if (notification is ScrollNotification) {
      //calculate
      int intIndexOfMiddleElement =
      (notification.metrics.pixels / itemExtent).round();
      intIndexOfMiddleElement = intIndexOfMiddleElement.clamp(0, stringSelections.length - 1);
      String intValueInTheMiddle = _valueFromIndex(intIndexOfMiddleElement + 1);

      if (_userStoppedScrolling(notification, intScrollController)) {
        //center selected value
        animateStringToIndex(intIndexOfMiddleElement);
      }

      //update selection
      if (intValueInTheMiddle != selectedValue) {
        String newValue;
        if (haptics) {
          HapticFeedback.selectionClick();
        }
        onChanged(newValue);
      }
    }
    return true;
  }


  ///There was a bug, when if there was small integer range, e.g. from 1 to 5,
  ///When user scrolled to the top, whole listview got displayed.
  ///To prevent this we are calculating cacheExtent by our own so it gets smaller if number of items is smaller
  double _calculateCacheExtent(int itemCount) {
    double cacheExtent = 250.0; //default cache extent
    if ((itemCount - 2) * kDefaultItemExtent <= cacheExtent) {
      cacheExtent = ((itemCount - 3) * kDefaultItemExtent);
    }
    return cacheExtent;
  }

  ///When overscroll occurs on iOS,
  ///we can end up with value not in the range between [minValue] and [maxValue]
  ///To avoid going out of range, we change values out of range to border values.
  int _normalizeMiddleValue(int valueInTheMiddle, int min, int max) {
    return math.max(math.min(valueInTheMiddle, max), min);
  }

  ///indicates if user has stopped scrolling so we can center value in the middle
  bool _userStoppedScrolling(
      Notification notification,
      ScrollController scrollController,
      ) {
    return notification is UserScrollNotification &&
        notification.direction == ScrollDirection.idle &&
        // ignore: invalid_use_of_protected_member, invalid_use_of_visible_for_testing_member
        scrollController.position.activity is! HoldScrollActivity;
  }

  ///scroll to selected value
  Future<void> _animate(ScrollController scrollController, double value) {
    return scrollController.animateTo(
      value,
      duration: Duration(seconds: 1),
      curve: ElasticOutCurve(),
    );
  }
}

class _NumberPickerSelectedItemDecoration extends StatelessWidget {
  final Axis axis;
  final double itemExtent;
  final Decoration decoration;

  const _NumberPickerSelectedItemDecoration(
      {Key key,
        @required this.axis,
        @required this.itemExtent,
        @required this.decoration})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Center(
      child: IgnorePointer(
        child: Container(
          width: isVertical ? double.infinity : itemExtent,
          height: isVertical ? itemExtent : double.infinity,
          decoration: decoration,
        ),
      ),
    );
  }

  bool get isVertical => axis == Axis.vertical;
}
