import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'dart:async';

import 'package:nui_core/data/nui_data_util.dart';

enum PageTransition{
  BOTTOM_TOP,
  LEFT_RIGHT,
  RIGHT_LEFT,
  SCALE,
  FADE,
  BLUR_BACKGROUND
}

typedef ContextGetter = BuildContext Function();
typedef NavigateToScreenCallback = void Function(Widget page, {PageTransition transition, Duration duration});

void popAndPush(BuildContext context, Widget page, {PageTransition transition, Duration duration}){
  NUINavigator.popAndPush(context, page, transition: transition, duration: duration);
}
void pop(BuildContext context){
  NUINavigator.pop(context);
}

void maybePop(BuildContext context){
  NUINavigator.maybePop(context);
}

void popAllAndPush(BuildContext context, Widget page, {PageTransition transition, Duration duration}) {
  NUINavigator.popAllAndPush(context, page, transition: transition, duration: duration);
}

void pushFull(BuildContext context, Widget page, {PageTransition transition, Duration duration}){
  NUINavigator.pushFull(context, page, transition: transition, duration: duration);
}

void push(BuildContext context, Widget page, {PageTransition transition, Duration duration}){
  NUINavigator.push(context, page, transition: transition, duration: duration);
}

Future<T> pushForResult<T>(BuildContext context, Widget page, {PageTransition transition, Duration duration}) async{
  return NUINavigator.pushForResult<T>(context, page, transition: transition, duration: duration);
}

void popWithResult<T>(BuildContext context, T data){
  NUINavigator.popWithResult(context, data);
}


class NUINavigator{
    NUINavigator._builder();

    static NavigateToScreenCallback _fullScreenNavigator;
    static List<String> _screenStack = List();
    static List<ContextGetter> _contextStack = List();

    static void addScreen(String code) => _screenStack.add(code);
    static void popScreen(String code) => _screenStack.remove(code);

    static void addContext(ContextGetter context) => _contextStack.add(context);
    static void popContext() => _contextStack.removeLast();

    static String getCurrentScreenCode() => _screenStack.length > 0 ? _screenStack.last : null;

    static BuildContext getLastContext(){
      if(isNullOrEmpty(_contextStack)) return null;
      try{
        return _contextStack.last();
      }catch(e){
        return null;
      }
    }

    static setLatestScreenCallback(NavigateToScreenCallback callback) => _fullScreenNavigator = callback;

    static Future<T> pushWithResult<T>(BuildContext context, Widget page) async {
      return await Navigator.push(context, MaterialPageRoute(
        builder: (context) => page),
      );
   }

   static void popWithResult<T>(BuildContext context, T data) => Navigator.pop(context, data);

    static dynamic _getPageRoute<T>(PageTransition transition, Duration duration, Widget page){
      dynamic route;
      switch(transition){
        case PageTransition.BOTTOM_TOP : {
          route = PageRouteBuilder<T>(
            transitionDuration: duration ?? Duration(milliseconds: 500),
            pageBuilder: (context, animation, secondaryAnimation) => page,
            transitionsBuilder: (context, animation, secondaryAnimation, child) {
              var begin = Offset(0.0, 1.0);
              var end = Offset.zero;
              var curve = Curves.ease;

              var tween = Tween(begin: begin, end: end).chain(CurveTween(curve: curve));
              return SlideTransition(position: animation.drive(tween), child: child);
            },
          );
          break;
        }
        case PageTransition.LEFT_RIGHT : {
          route = PageRouteBuilder<T>(
            pageBuilder: (context, animation, secondaryAnimation,) =>
            page,
            transitionsBuilder: (context, animation, secondaryAnimation, child) =>
                SlideTransition(
                  position: Tween<Offset>(
                    begin: const Offset(-1, 0),
                    end: Offset.zero,
                  ).animate(animation),
                  child: child,
                ),
          );
          break;
        }
        case PageTransition.BLUR_BACKGROUND : {
          route = PageRouteBuilder<T>(
            pageBuilder: (context, animation, secondaryAnimation,) =>
            page,
            transitionsBuilder: (context, animation, secondaryAnimation, child) =>
                BackdropFilter(
                  filter: ImageFilter.blur(sigmaX: 4 * animation.value, sigmaY: 4 * animation.value),
                  child: FadeTransition(
                    child: child,
                    opacity: animation,
                  ),
                ),
          );
          break;
        }
        case PageTransition.RIGHT_LEFT : {
          route = PageRouteBuilder<T>(
            pageBuilder: (context, animation, secondaryAnimation,) =>
            page,
            transitionsBuilder: (context, animation, secondaryAnimation, child) =>
                SlideTransition(
                  position: Tween<Offset>(
                    begin: const Offset(1, 0),
                    end: Offset.zero,
                  ).animate(animation),
                  child: child,
                ),
          );
          break;
        }
        case PageTransition.SCALE : {
          route = PageRouteBuilder<T>(
            pageBuilder: (context, animation, secondaryAnimation,) =>
            page,
            transitionsBuilder: (context, animation, secondaryAnimation, child) =>
                ScaleTransition(
                  scale: Tween<double>(
                    begin: 0.0,
                    end: 1.0,
                  ).animate(
                    CurvedAnimation(
                      parent: animation,
                      curve: Curves.fastOutSlowIn,
                    ),
                  ),
                  child: child,
                ),
          );
          break;
        }
        case PageTransition.FADE : {
          route = PageRouteBuilder<T>(
            pageBuilder: (context, animation, secondaryAnimation,) =>
            page,
            transitionsBuilder: (context, animation, secondaryAnimation, child) =>
                FadeTransition(
                  opacity: animation,
                  child: child,
                ),
          );
          break;
        }
        default: {
          route = MaterialPageRoute<T>(
              builder: (context) => page
          );
        }
      }
      return route;
    }

   static _pushWithTransition(BuildContext context, Widget page, {PageTransition transition, Duration duration, bool replace: false}){
      final route = _getPageRoute(transition, duration, page);
      if(replace){
        Navigator.of(context).pushReplacement(route);
      }
      else {
        Navigator.of(context).push(route);
      }
   }

   static Future<T> _pushForResult<T>(BuildContext context, Widget page, {PageTransition transition, Duration duration, bool replace: false}) async{
     final route = _getPageRoute<T>(transition, duration, page);
     T result;
     if(replace){
       result = await Navigator.of(context).pushReplacement(route);
     }
     else {
       result = await Navigator.of(context).push(route);
     }
     return result;
   }

   static void push(BuildContext context, Widget page, {PageTransition transition, Duration duration}) => _pushWithTransition(context, page, transition: transition, duration: duration);

    static Future<T> pushForResult<T>(BuildContext context, Widget page, {PageTransition transition, Duration duration}) => _pushForResult<T>(context, page, transition: transition, duration: duration);

    static void pushFull(BuildContext context, Widget page, {PageTransition transition, Duration duration}){
      try{
        _fullScreenNavigator(page, transition: transition, duration: duration);
      }catch(e){
        _pushWithTransition(context, page, transition: transition, duration: duration);
      }
    }

    static void maybePop(BuildContext context, {bool navigatorPop: true}) async{
      bool popSuccess = await Navigator.of(context).maybePop();
      if (!popSuccess && navigatorPop == true) {
        SystemNavigator.pop();
      }
    }

    static void popAllAndPush(BuildContext context, Widget page, {PageTransition transition, Duration duration}){
      final route = _getPageRoute(transition, duration, page);
      Navigator.of(context).pushAndRemoveUntil(route, (Route<dynamic> route) => false);
    }

    static void popAndPush(BuildContext context, Widget page, {PageTransition transition, Duration duration}){
       _pushWithTransition(context, page, transition: transition, duration: duration, replace: true);
    }

    static void pop(BuildContext context) => Navigator.pop(context);

}